/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

/*
 * Timestamp.cpp
 *
 *  Created on: Jul 22, 2009
 *      Author: jwienke
 */

#include "Timestamp.h"

#include <sstream>
#include <cmath>

#include <sys/time.h>
#include <time.h>

using namespace xmltio;
using namespace std;

namespace airobots {

std::string Timestamp::getBaseTag() {
	return "TIMESTAMP";
}

std::string Timestamp::getTemplate() {
	return "<" + getBaseTag()
			+ "><INSERTED value=\"\"/><UPDATED value=\"\"/></" + getBaseTag()
			+ ">";
}

uint64_t Timestamp::currentTimeMillis() {
	timeval tv;
	gettimeofday(&tv, NULL);

	// (second-value in milliseconds) + (microsecond value in milliseconds)
	return (((uint64_t) tv.tv_sec) * 1000l) + (tv.tv_usec / 1000l);
}

bool Timestamp::operator==(const Timestamp &other) const {
	return (inserted == other.inserted) && (updated == other.updated);
}

uint64_t Timestamp::getInserted() const {
	return inserted;
}

void Timestamp::setInserted(uint64_t inserted) {
	this->inserted = inserted;
}

uint64_t Timestamp::getUpdated() const {
	return updated;
}

void Timestamp::setUpdated(uint64_t updated) {
	this->updated = updated;
}

ostream& operator<<(ostream &o, Timestamp const &timestamp) {
	o << "Timestamp(inserted = " << timestamp.getInserted() << ", updated = "
			<< timestamp.getUpdated() << ")";
	return o;
}
}

template<>
airobots::Timestamp xmltio::extract<airobots::Timestamp>::get() const {

	stringstream insertedStream;
	stringstream updatedStream;

	string inserted = extract<string> (loc_[XPath("INSERTED/@value")]);
	insertedStream << inserted;
	uint64_t insertedTime;
	insertedStream >> insertedTime;
	string updated = extract<string> (loc_[XPath("UPDATED/@value")]);
	updatedStream << updated;
	uint64_t updatedTime;
	updatedStream >> updatedTime;

	return airobots::Timestamp(insertedTime, updatedTime);
}

void xmltio::convert(const airobots::Timestamp& timestamp, Location& loc) {

	stringstream insertedStream;
	stringstream updatedStream;
	insertedStream << timestamp.getInserted();
	updatedStream << timestamp.getUpdated();

	loc[XPath("INSERTED/@value")] = insertedStream.str();
	loc[XPath("UPDATED/@value")] = updatedStream.str();

}
