/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

/*
 * Timestamp.h
 *
 *  Created on: Jul 22, 2009
 *      Author: jwienke
 */

#ifndef TIMESTAMP_H_
#define TIMESTAMP_H_

#include <string>
#include <sstream>
#include <iostream>
#include <stdint.h>

#include <xmltio/xmltio.hpp>

namespace airobots {

/**
 * Timestamp associated with data types in milliseconds unix time.
 *
 * @author jwienke
 */
class Timestamp {
public:

	/**
	 * Returns the current system time in unix format as milliseconds.
	 */
	static uint64_t currentTimeMillis();

	inline Timestamp() :
		inserted(currentTimeMillis()), updated(inserted) {
	}

	inline Timestamp(uint64_t inserted) :
		inserted(inserted), updated(inserted) {
	}

	inline Timestamp(uint64_t inserted, uint64_t updated) :
		inserted(inserted), updated(updated) {
	}

	static std::string getBaseTag();

	static std::string getTemplate();

	bool operator==(const Timestamp &other) const;

	uint64_t getInserted() const;
	void setInserted(uint64_t inserted);

	uint64_t getUpdated() const;
	void setUpdated(uint64_t updated);

private:
	/**
	 * Represents the inserting time of the object.
	 */
	uint64_t inserted;

	/**
	 * Represents the last time of update of that object.
	 */
	uint64_t updated;

};

std::ostream& operator<<(std::ostream& o, Timestamp const& timestamp);

}

namespace xmltio {

void convert(const airobots::Timestamp& timestamp, Location& loc);

template<>
airobots::Timestamp extract<airobots::Timestamp>::get() const;

}

#endif /* TIMESTAMP_H_ */
