/*
 * This file is part of Bielefeld Type Library (BTL).
 *
 * Copyright(c) Frederic Siepmann
 * http://opensource.cit-ec.de/projects/btl
 *
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 *
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
 * 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
 * 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke,
 *               Lukas Kettenbach
 */

/* 
 * File:   Annotation.cpp
 * Author: lkettenb
 * 
 * Created on April 18, 2012, 11:31 AM
 */

#include "Annotation.h"

airobots::Annotation::Annotation(std::string _label,
        airobots::PrecisePolygon _polygon,
        std::list<airobots::Viewpoint> _viewpoints)
:
label(_label),
polygon(_polygon),
viewpoints(_viewpoints) {

}

std::string airobots::Annotation::getJavaClassName() {
    return airobots::Type::JAVA_CLASS_NAME_PREFIX + "data.map.Annotation";
}

std::string airobots::Annotation::getBaseTag() {
    return "ANNOTATION";
}

std::string airobots::Annotation::getTemplate() {
    return "<" + getBaseTag() + " label=\"\" >" +
            airobots::Timestamp::getTemplate() + 
            "<GENERATOR/>" +
            "</" + getBaseTag() + ">";
}

void airobots::Annotation::fillInto(xmltio::Location &parent) const {
    airobots::Type::fillInto(parent);
    
    // Set attributes.
    parent[xmltio::XPath("@label")] = label;
    
    // Set viewpoints.
    std::list<airobots::Viewpoint>::const_iterator it;
    for (it = viewpoints.begin(); it != viewpoints.end(); ++it) {
        xmltio::Location item(airobots::Viewpoint::getTemplate(),
            airobots::Viewpoint::getBaseTag());
        item = *it;
        parent.add(item);
    }
    
    // Set polygon
    xmltio::Location item(airobots::PrecisePolygon::getTemplate(),
            airobots::PrecisePolygon::getBaseTag());
    item = polygon;
    parent.add(item);
}

void airobots::Annotation::fromLocation(const xmltio::Location &location,
            Annotation &annotation) {
    // Get label.    
    annotation.setLabel(xmltio::extract<std::string>(location[xmltio::XPath("@label")]).get());
    
    // Get viewpoints.
    std::list<airobots::Viewpoint> tmpViewpoints;
	for (xmltio::XPathIterator i = 
            xmltio::XPath(airobots::Viewpoint::getBaseTag()).evaluate(location); 
            i; 
            ++i) {
		xmltio::Location viewpointLocation(*i);
        airobots::Viewpoint vp = 
                xmltio::extract<airobots::Viewpoint>(viewpointLocation);
        tmpViewpoints.push_back(vp);
	}
    annotation.setViewpoints(tmpViewpoints);
    
    // Get polygon.
	for (xmltio::XPathIterator i = 
            xmltio::XPath(airobots::PrecisePolygon::getBaseTag()).evaluate(location); 
            i; 
            ++i) {
		xmltio::Location polygonLocation(*i);
        airobots::PrecisePolygon p = 
                xmltio::extract<airobots::PrecisePolygon>(polygonLocation);
        annotation.setPolygon(p);
	}
}

void airobots::Annotation::sanitizeLocation(xmltio::Location &location) const {
    xmltio::XPath xp(getBaseTag());
	while (xmltio::XPathIterator i = xp.evaluate(location)) {
		xmltio::Location l(*i);
		location.remove(l);
	}
}
