/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

/*
 * ROSBaseLocalPlannerConfiguration.h
 *
 *  Created on: Jun 13, 2012
 *      Author: pdressel
 */

#ifndef ROSBASELOCALPLANNERCONFIGURATION_H_
#define ROSBASELOCALPLANNERCONFIGURATION_H_

#include <stdio.h>
#include <string>
#include <xmltio/xmltio.hpp>
#include "ROSDynamicConfiguration.h"

namespace airobots 
{

class ROSBaseLocalPlannerConfiguration : public ROSDynamicConfiguration 
{
    
public:
    
    /**
     * ROSBaseLocalPlannerConfiguration constructor. Make sure the member variables are 
     * set with the current configuration as is on the corresponding ROS server.
     */
    ROSBaseLocalPlannerConfiguration();
    virtual ~ROSBaseLocalPlannerConfiguration() {}

    /**
     * Explanation: The x acceleration limit of the robot in meters/sec^2.<br>
     * name: '<node_name>/acc_lim_x'.<br>
     * Type and default: double, 2.5.
     * @return the acc_lim_x
     */
    double getAccLimX() {
        return this->acc_lim_x;
    }

    /**
     * Explanation: The x acceleration limit of the robot in meters/sec^2.<br>
     * name: '<node_name>/acc_lim_x'.<br>
     * Type and default: double, 2.5.
     * @param acc_lim_x the acc_lim_x to set
     */
    void setAccLimX(double acc_lim_x) {
        this->acc_lim_x = acc_lim_x;
    }

    /**
     * Explanation: The y acceleration limit of the robot in meters/sec^2.<br>
     * name: '<node_name>/acc_lim_y'.<br>
     * Type and default: double, 2.5.
     * @return the acc_lim_y
     */
    double getAccLimY() {
        return this->acc_lim_y;
    }

    /**
     * Explanation: The y acceleration limit of the robot in meters/sec^2.<br>
     * name: '<node_name>/acc_lim_y'.<br>
     * Type and default: double, 2.5.
     * @param acc_lim_y the acc_lim_y to set
     */
    void setAccLimY(double acc_lim_y) {
        this->acc_lim_y = acc_lim_y;
    }

    /**
     * Explanation: The rotational acceleration limit of the robot in
     * radians/sec^2 .<br>
     * name: '<node_name>/acc_lim_th'.<br>
     * Type and default: double, 3.2.
     * @return the acc_lim_theta
     */
    double getAccLimTheta() {
        return this->acc_lim_theta;
    }

    /**
     * Explanation: The rotational acceleration limit of the robot in
     * radians/sec^2 .<br>
     * name: '<node_name>/acc_lim_th'.<br>
     * Type and default: double, 3.2.
     * @param acc_lim_theta the acc_lim_theta to set
     */
    void setAccLimTheta(double acc_lim_theta) {
        this->acc_lim_theta = acc_lim_theta;
    }

    /**
     * Explanation: The maximum forward velocity allowed for the base in
     * meters/sec.<br>
     * name: '<node_name>/max_vel_x'.<br>
     * Type and default: double, 0.5.
     * @return the max_vel_x
     */
    double getMaxVelX() {
        return this->max_vel_x;
    }

    /**
     * Explanation: The maximum forward velocity allowed for the base in
     * meters/sec.<br>
     * name: '<node_name>/max_vel_x'.<br>
     * Type and default: double, 0.5.
     * @param max_vel_x the max_vel_x to set
     */
    void setMaxVelX(double max_vel_x) {
        this->max_vel_x = max_vel_x;
    }

    /**
     * Explanation: The minimum forward velocity allowed for the base in
     * meters/sec. It is useful to specify this to guarantee that velocity
     * commands sent to a mobile base are high enough to allow the base to
     * overcome friction.<br>
     * name: '<node_name>/min_vel_x'.<br>
     * Type and default: double, 0.1.
     * @return the min_vel_x
     */
    double getMinVelX() {
        return this->min_vel_x;
    }

    /**
     * Explanation: The minimum forward velocity allowed for the base in
     * meters/sec. It is useful to specify this to guarantee that velocity
     * commands sent to a mobile base are high enough to allow the base to
     * overcome friction.<br>
     * name: '<node_name>/min_vel_x'.<br>
     * Type and default: double, 0.1.
     * @param min_vel_x the min_vel_x to set
     */
    void setMinVelX(double min_vel_x) {
        this->min_vel_x = min_vel_x;
    }

    /**
     * Explanation: The maximum rotational velocity allowed for the base in
     * radians/sec.<br>
     * name: '<node_name>/max_rotational_vel'.<br>
     * Type and default: double, 1.0.
     * @return the max_vel_theta
     */
    double getMaxVelTheta() {
        return this->max_vel_theta;
    }

    /**
     * Explanation: The maximum rotational velocity allowed for the base in
     * radians/sec.<br>
     * name: '<node_name>/max_rotational_vel'.<br>
     * Type and default: double, 1.0.
     * @param max_vel_theta the max_vel_theta to set
     */
    void setMaxVelTheta(double max_vel_theta) {
        this->max_vel_theta = max_vel_theta;
    }

    /**
     * Explanation: The minimum rotational velocity allowed for the base in
     * radians/sec.<br>
     * name: '<node_name>/min_rotational_vel'.<br>
     * Type and default: double, 0.0.
     * @return the min_vel_theta
     */
    double getMinVelTheta() {
        return this->min_vel_theta;
    }

    /**
     * Explanation: The minimum rotational velocity allowed for the base in
     * radians/sec.<br>
     * name: '<node_name>/min_rotational_vel'.<br>
     * Type and default: double, 0.0.
     * @param min_vel_theta the min_vel_theta to set
     */
    void setMinVelTheta(double min_vel_theta) {
        this->min_vel_theta = min_vel_theta;
    }

    /**
     * Explanation: The minimum rotational velocity allowed for the base while
     * performing in-place rotations in radians/sec.<br>
     * name: '<node_name>/min_in_place_rotational_vel'.<br>
     * Type and default: double, 0.4.
     * @return the min_in_place_vel_theta
     */
    double getMinInPlaceVelTheta() {
        return this->min_in_place_vel_theta;
    }

    /**
     * Explanation: The minimum rotational velocity allowed for the base while
     * performing in-place rotations in radians/sec.<br>
     * name: '<node_name>/min_in_place_rotational_vel'.<br>
     * Type and default: double, 0.4.
     * @param min_in_place_vel_theta the min_in_place_vel_theta to set
     */
    void setMinInPlaceVelTheta(double min_in_place_vel_theta) {
        this->min_in_place_vel_theta = min_in_place_vel_theta;
    }

    /**
     * Explanation: The amount of time to forward-simulate trajectories in seconds.<br>
     * name: '<node_name>/sim_time'.<br>
     * Type and default: double, 1.0.
     * @return the sim_time
     */
    double getSimTime() {
        return this->sim_time;
    }

    /**
     * Explanation: The amount of time to forward-simulate trajectories in seconds.<br>
     * name: '<node_name>/sim_time'.<br>
     * Type and default: double, 1.0.
     * @param sim_time the sim_time to set
     */
    void setSimTime(double sim_time) {
        this->sim_time = sim_time;
    }

    /**
     * Explanation: The step size, in meters, to take between points on a given
     * trajectory.<br>
     * name: '<node_name>/sim_granularity'.<br>
     * Type and default: double, 0.025.
     * @return the sim_granularity
     */
    double getSimGranularity() {
        return this->sim_granularity;
    }

    /**
     * Explanation: The step size, in meters, to take between points on a given
     * trajectory.<br>
     * name: '<node_name>/sim_granularity'.<br>
     * Type and default: double, 0.025.
     * @param sim_granularity the sim_granularity to set
     */
    void setSimGranularity(double sim_granularity) {
        this->sim_granularity = sim_granularity;
    }

    /**
     * Explanation: The weighting for how much the controller should stay close
     * to the path it was given.<br>
     * name: '<node_name>/pdist_scale'.<br>
     * Type and default: double, 0.6.
     * @return the pdist_scale
     */
    double getPdistScale() {
        return this->pdist_scale;
    }

    /**
     * Explanation: The weighting for how much the controller should stay close
     * to the path it was given.<br>
     * name: '<node_name>/pdist_scale'.<br>
     * Type and default: double, 0.6.
     * @param pdist_scale the pdist_scale to set
     */
    void setPdistScale(double pdist_scale) {
        this->pdist_scale = pdist_scale;
    }

    /**
     * Explanation: The weighting for how much the controller should attempt to
     * reach its local goal, also controls speed.<br>
     * name: '<node_name>/gdist_scale'.<br>
     * Type and default: double, 0.8.
     * @return the gdist_scale
     */
    double getGdistScale() {
        return this->gdist_scale;
    }

    /**
     * Explanation: The weighting for how much the controller should attempt to
     * reach its local goal, also controls speed.<br>
     * name: '<node_name>/gdist_scale'.<br>
     * Type and default: double, 0.8.
     * @param gdist_scale the gdist_scale to set
     */
    void setGdistScale(double gdist_scale) {
        this->gdist_scale = gdist_scale;
    }

    /**
     * Explanation: The weighting for how much the controller should attempt to
     * avoid obstacles.<br>
     * name: '<node_name>/occdist_scale'.<br>
     * Type and default: double, 0.01.
     * @return the occdist_scale
     */
    double getOccdistScale() {
        return this->occdist_scale;
    }

    /**
     * Explanation: The weighting for how much the controller should attempt to
     * avoid obstacles.<br>
     * name: '<node_name>/occdist_scale'.<br>
     * Type and default: double, 0.01.
     * @param occdist_scale the occdist_scale to set
     */
    void setOccdistScale(double occdist_scale) {
        this->occdist_scale = occdist_scale;
    }

    /**
     * Explanation: How far the robot must travel in meters before oscillation
     * flags are reset.<br>
     * name: '<node_name>/oscillation_reset_dist'.<br>
     * Type and default: double, 0.05.
     * @return the oscillation_reset_dist
     */
    double getOscillationResetDist() {
        return this->oscillation_reset_dist;
    }

    /**
     * Explanation: How far the robot must travel in meters before oscillation
     * flags are reset.<br>
     * name: '<node_name>/oscillation_reset_dist'.<br>
     * Type and default: double, 0.05.
     * @param oscillation_reset_dist the oscillation_reset_dist to set
     */
    void setOscillationResetDist(double oscillation_reset_dist) {
        this->oscillation_reset_dist = oscillation_reset_dist;
    }

    /**
     * Explanation: The distance the robot must travel before it can exit escape
     * mode.<br>
     * name: '<node_name>/escape_reset_dist'.<br>
     * Type and default: double, 0.1.
     * @return the escape_reset_dist
     */
    double getEscapeResetDist() {
        return this->escape_reset_dist;
    }

    /**
     * Explanation: The distance the robot must travel before it can exit escape
     * mode.<br>
     * name: '<node_name>/escape_reset_dist'.<br>
     * Type and default: double, 0.1.
     * @param escape_reset_dist the escape_reset_dist to set
     */
    void setEscapeResetDist(double escape_reset_dist) {
        this->escape_reset_dist = escape_reset_dist;
    }

    /**
     * Explanation: The distance the robot must rotate before it can exit escape
     * mode.<br>
     * name: '<node_name>/escape_reset_theta'.<br>
     * Type and default: double, 1.5708 (PI/2).
     * @return the escape_reset_theta
     */
    double getEscapeResetTheta() {
        return this->escape_reset_theta;
    }

    /**
     * Explanation: The distance the robot must rotate before it can exit escape
     * mode.<br>
     * name: '<node_name>/escape_reset_theta'.<br>
     * Type and default: double, 1.5708 (PI/2).
     * @param escape_reset_theta the escape_reset_theta to set
     */
    void setEscapeResetTheta(double escape_reset_theta) {
        this->escape_reset_theta = escape_reset_theta;
    }

    /**
     * Explanation: The number of samples to use when exploring the x velocity
     * space.<br>
     * name: '<node_name>/vx_samples'.<br>
     * Type and default: integer, 3.
     * @return the vx_samples
     */
    int getVxSamples() {
        return this->vx_samples;
    }

    /**
     * Explanation: The number of samples to use when exploring the x velocity
     * space.<br>
     * name: '<node_name>/vx_samples'.<br>
     * Type and default: integer, 3.
     * @param vx_samples the vx_samples to set
     */
    void setVxSamples(int vx_samples) {
        this->vx_samples = vx_samples;
    }

    /**
     * Explanation: The number of samples to use when exploring the theta velocity space .<br>
     * name: '<node_name>/vtheta_samples'.<br>
     * Type and default: integer, 20.
     * @return the vtheta_samples
     */
    int getVthetaSamples() {
        return this->vtheta_samples;
    }

    /**
     * Explanation: The number of samples to use when exploring the theta velocity space .<br>
     * name: '<node_name>/vtheta_samples'.<br>
     * Type and default: integer, 20.
     * @param vtheta_samples the vtheta_samples to set
     */
    void setVthetaSamples(int vtheta_samples) {
        this->vtheta_samples = vtheta_samples;
    }

    /**
     * Explanation: How far to look ahead in meters when scoring different
     * in-place-rotation trajectories.<br>
     * name: '<node_name>/heading_lookahead'.<br>
     * Type and default: double, 0.325.
     * @return the heading_lookahead
     */
    double getHeadingLookahead() {
        return this->heading_lookahead;
    }

    /**
     * Explanation: How far to look ahead in meters when scoring different
     * in-place-rotation trajectories.<br>
     * name: '<node_name>/heading_lookahead'.<br>
     * Type and default: double, 0.325.
     * @param heading_lookahead the heading_lookahead to set
     */
    void setHeadingLookahead(double heading_lookahead) {
        this->heading_lookahead = heading_lookahead;
    }

    /**
     * Explanation: Determines whether velocity commands are generated for a
     * holonomic or non-holonomic robot. For holonomic robots, strafing velocity
     * commands may be issued to the base. For non-holonomic robots, no strafing
     * velocity commands will be issued.<br>
     * name: '<node_name>/holonomic_robot'.<br>
     * Type and default: bool, true.
     * @return the holonomic_robot
     */
    bool isHolonomicRobot() {
        return this->holonomic_robot;
    }

    /**
     * Explanation: Determines whether velocity commands are generated for a
     * holonomic or non-holonomic robot. For holonomic robots, strafing velocity
     * commands may be issued to the base. For non-holonomic robots, no strafing
     * velocity commands will be issued.<br>
     * name: '<node_name>/holonomic_robot'.<br>
     * Type and default: bool, true.
     * @param holonomic_robot the holonomic_robot to set
     */
    void setHolonomicRobot(bool holonomic_robot) {
        this->holonomic_robot = holonomic_robot;
    }

    /**
     * Explanation: Speed used for driving during escapes in meters/sec. Note
     * that it must be negative in order for the robot to actually reverse. A
     * positive speed will cause the robot to move forward while attempting to
     * escape.<br>
     * name: '<node_name>/escape_vel'.<br>
     * Type and default: double, -0.1.
     * @return the escape_vel
     */
    double getEscapeVel() {
        return this->escape_vel;
    }

    /**
     * Explanation: Speed used for driving during escapes in meters/sec. Note
     * that it must be negative in order for the robot to actually reverse. A
     * positive speed will cause the robot to move forward while attempting to
     * escape.<br>
     * name: '<node_name>/escape_vel'.<br>
     * Type and default: double, -0.1.
     * @param escape_vel the escape_vel to set
     */
    void setEscapeVel(double escape_vel) {
        this->escape_vel = escape_vel;
    }

    /**
     * Explanation: Whether to use the Dynamic Window Approach (DWA)_ or whether
     * to use Trajectory Rollout (NOTE: In our experience DWA worked as well as
     * Trajectory Rollout and is computationally less expensive. It is possible
     * that robots with extremely poor acceleration limits could gain from
     * running Trajectory Rollout).<br>
     * name: '<node_name>/dwa'.<br>
     * Type and default: bool, true.
     * @return the dwa
     */
    bool isDwa() {
        return this->dwa;
    }

    /**
     * Explanation: Whether to use the Dynamic Window Approach (DWA)_ or whether
     * to use Trajectory Rollout (NOTE: In our experience DWA worked as well as
     * Trajectory Rollout and is computationally less expensive. It is possible
     * that robots with extremely poor acceleration limits could gain from
     * running Trajectory Rollout).<br>
     * name: '<node_name>/dwa'.<br>
     * Type and default: bool, true.
     * @param dwa the dwa to set
     */
    void setDwa(bool dwa) {
        this->dwa = dwa;
    }

    /**
     * Explanation: Whether to score based on the robot's heading to the path
     * or its distance from the path.<br>
     * name: '<node_name>/heading_scoring'.<br>
     * Type and default: bool, false.
     * @return the heading_scoring
     */
    bool isHeadingScoring() {
        return this->heading_scoring;
    }

    /**
     * Explanation: Whether to score based on the robot's heading to the path
     * or its distance from the path.<br>
     * name: '<node_name>/heading_scoring'.<br>
     * Type and default: bool, false.
     * @param heading_scoring the heading_scoring to set
     */
    void setHeadingScoring(bool heading_scoring) {
        this->heading_scoring = heading_scoring;
    }

    /**
     * Explanation: How far to look ahead in time in seconds along the simulated
     * trajectory when using heading scoring.<br>
     * name: '<node_name>/heading_scoring_timestep'.<br>
     * Type and default: double, 0.8.
     * @return the heading_scoring_timestep
     */
    double getHeadingScoringTimestep() {
        return this->heading_scoring_timestep;
    }

    /**
     * Explanation: How far to look ahead in time in seconds along the simulated
     * trajectory when using heading scoring.<br>
     * name: '<node_name>/heading_scoring_timestep'.<br>
     * Type and default: double, 0.8.
     * @param heading_scoring_timestep the heading_scoring_timestep to set
     */
    void setHeadingScoringTimestep(double heading_scoring_timestep) {
        this->heading_scoring_timestep = heading_scoring_timestep;
    }

    /**
     * Explanation: Set this to true to allow simple attraction to a goal point
     * instead of intelligent cost propagation.<br>
     * name: '<node_name>/planner_frequency'.<br>
     * Type and default: bool, false.
     * @return the simple_attractor
     */
    bool isSimpleAttractor() {
        return this->simple_attractor;
    }

    /**
     * Explanation: Set this to true to allow simple attraction to a goal point
     * instead of intelligent cost propagation.<br>
     * name: '<node_name>/planner_frequency'.<br>
     * Type and default: bool, false.
     * @param simple_attractor the simple_attractor to set
     */
    void setSimpleAttractor(bool simple_attractor) {
        this->simple_attractor = simple_attractor;
    }

    /**
     * Explanation: The step size, in radians, to take between angular samples
     * on a given trajectory.<br>
     * name: '<node_name>/angular_sim_granularity'.<br>
     * Type and default: double, 0.025.
     * @return the angular_sim_granularity
     */
    double getAngularSimGranularity() {
        return this->angular_sim_granularity;
    }

    /**
     * Explanation: The step size, in radians, to take between angular samples
     * on a given trajectory.<br>
     * name: '<node_name>/angular_sim_granularity'.<br>
     * Type and default: double, 0.025.
     * @param angular_sim_granularity the angular_sim_granularity to set
     */
    void setAngularSimGranularity(double angular_sim_granularity) {
        this->angular_sim_granularity = angular_sim_granularity;
    }

    /**
     * Explanation: The strafing velocities that a holonomic robot will consider in meters/sec.<br>
     * name: '<node_name>/y_vels'.<br>
     * Type and default: list, '[-0.3, -0.1, 0.1, 0.3]'.
     * @return the y_vels
     */ 
    std::string getYVels() {
        return this->y_vels;
    }

    /**
     * Explanation: The strafing velocities that a holonomic robot will consider in meters/sec.<br>
     * name: '<node_name>/y_vels'.<br>
     * Type and default: list, '[-0.3, -0.1, 0.1, 0.3]'.
     * @param y_vels the y_vels to set
     */
    void setYVels(std::string y_vels) {
        this->y_vels = y_vels;
    }

    /**
     * Explanation: Reset all values at the target dynamic_reconfiguration
     * server to its defaults. ROS topic name: '<node_name>/restore_defaults'.
     * Type and default: bool, false
     * @return the restore_defaults
     */
    bool isRestoreDefaults() {
        return this->restore_defaults;
    }

    /**
     * Explanation: Reset all values at the target dynamic_reconfiguration
     * server to its defaults. ROS topic name: '<node_name>/restore_defaults'.
     * Type and default: bool, false
     * @param restore_defaults the restore_defaults to set
     */
    void setRestoreDefaults(bool restore_defaults) {
        this->restore_defaults = restore_defaults;
    }
    
    static std::string getBaseTag();
    static std::string getTemplate();    
    
    void sanitizeLocation(xmltio::Location &loc) const;
    void fillInto(xmltio::Location &loc) const;
    static void fromLocation(const xmltio::Location &loc, ROSBaseLocalPlannerConfiguration &config);
        
    static const std::string ACC_LIM_X_ATTRIBUTE;   
    static const std::string ACC_LIM_Y_ATTRIBUTE;
    static const std::string ACC_LIM_THETA_ATTRIBUTE;
    static const std::string MAX_VEL_X_ATTRIBUTE;
    static const std::string MIN_VEL_X_ATTRIBUTE;
    static const std::string MAX_VEL_THETA_ATTRIBUTE;
    static const std::string MIN_VEL_THETA_ATTRIBUTE;
    static const std::string MIN_IN_PLACE_VEL_THETA_ATTRIBUTE;
    static const std::string SIM_TIME_ATTRIBUTE;
    static const std::string SIM_GRANULARITY_ATTRIBUTE;
    static const std::string PDIST_SCALE_ATTRIBUTE;
    static const std::string GDIST_SCALE_ATTRIBUTE;
    static const std::string OCCDIST_SCALE_ATTRIBUTE;
    static const std::string OSCILLATION_RESET_DIST_ATTRIBUTE;
    static const std::string ESCAPE_RESET_DIST_ATTRIBUTE;
    static const std::string ESCAPE_RESET_THETA_ATTRIBUTE;
    static const std::string VX_SAMPLES_ATTRIBUTE;
    static const std::string VTHETA_SAMPLES_ATTRIBUTE;
    static const std::string HEADING_LOOKAHEAD_ATTRIBUTE;
    static const std::string HOLONOMIC_ROBOT_ATTRIBUTE;
    static const std::string ESCAPE_VEL_ATTRIBUTE;
    static const std::string DWA_ATTRIBUTE;
    static const std::string HEADING_SCORING_ATTRIBUTE;
    static const std::string HEADING_SCORING_TIMESTAMP_ATTRIBUTE;
    static const std::string SIMPLE_ATTRACTOR_ATTRIBUTE;
    static const std::string ANGULAR_SIM_GRANULARITY_ATTRIBUTE;
    static const std::string Y_VELS_ATTRIBUTE;
    static const std::string RESTORE_DEFAULTS_ATTRIBUTE;
     
private:
    
    static const std::string BASELOCALPLANNER_CONFIGURATION_TAG;
    
    /**
     * Explanation: The x acceleration limit of the robot in meters/sec^2.<br>
     * name: '<node_name>/acc_lim_x'.<br>
     * Type and default: double, 2.5.
     */
    double acc_lim_x;
    
    /**
     * Explanation: The y acceleration limit of the robot in meters/sec^2.<br>
     * name: '<node_name>/acc_lim_y'.<br>
     * Type and default: double, 2.5.
     */
    double acc_lim_y;

    /**
     * Explanation: The rotational acceleration limit of the robot in 
     * radians/sec^2 .<br>
     * name: '<node_name>/acc_lim_th'.<br>
     * Type and default: double, 3.2.
     */
    double acc_lim_theta;

    /**
     * Explanation: The maximum forward velocity allowed for the base in 
     * meters/sec.<br>
     * name: '<node_name>/max_vel_x'.<br>
     * Type and default: double, 0.5.
     */
    double max_vel_x;

    /**
     * Explanation: The minimum forward velocity allowed for the base in 
     * meters/sec. It is useful to specify this to guarantee that velocity 
     * commands sent to a mobile base are high enough to allow the base to 
     * overcome friction.<br>
     * name: '<node_name>/min_vel_x'.<br>
     * Type and default: double, 0.1.
     */
    double min_vel_x;

    /**
     * Explanation: The maximum rotational velocity allowed for the base in 
     * radians/sec.<br>
     * name: '<node_name>/max_rotational_vel'.<br>
     * Type and default: double, 1.0.
     */
    double max_vel_theta;

    /**
     * Explanation: The minimum rotational velocity allowed for the base in 
     * radians/sec.<br>
     * name: '<node_name>/min_rotational_vel'.<br>
     * Type and default: double, 0.0.
     */
    double min_vel_theta;

    /**
     * Explanation: The minimum rotational velocity allowed for the base while 
     * performing in-place rotations in radians/sec.<br>
     * name: '<node_name>/min_in_place_rotational_vel'.<br>
     * Type and default: double, 0.4.
     */    
    double min_in_place_vel_theta;

    /**
     * Explanation: The amount of time to forward-simulate trajectories in seconds.<br>
     * name: '<node_name>/sim_time'.<br>
     * Type and default: double, 1.0.
     */    
    double sim_time;

    /**
     * Explanation: The step size, in meters, to take between points on a given 
     * trajectory.<br>
     * name: '<node_name>/sim_granularity'.<br>
     * Type and default: double, 0.025.
     */    
    double sim_granularity;
    
    /**
     * Explanation: The weighting for how much the controller should stay close 
     * to the path it was given.<br>
     * name: '<node_name>/pdist_scale'.<br>
     * Type and default: double, 0.6.
     */    
    double pdist_scale;

    /**
     * Explanation: The weighting for how much the controller should attempt to 
     * reach its local goal, also controls speed.<br>
     * name: '<node_name>/gdist_scale'.<br>
     * Type and default: double, 0.8.
     */    
    double gdist_scale;

    /**
     * Explanation: The weighting for how much the controller should attempt to 
     * avoid obstacles.<br>
     * name: '<node_name>/occdist_scale'.<br>
     * Type and default: double, 0.01.
     */    
    double occdist_scale;

    /**
     * Explanation: How far the robot must travel in meters before oscillation 
     * flags are reset.<br>
     * name: '<node_name>/oscillation_reset_dist'.<br>
     * Type and default: double, 0.05.
     */    
    double oscillation_reset_dist;

    /**
     * Explanation: The distance the robot must travel before it can exit escape
     * mode.<br>
     * name: '<node_name>/escape_reset_dist'.<br>
     * Type and default: double, 0.1.
     */
    double escape_reset_dist;

    /**
     * Explanation: The distance the robot must rotate before it can exit escape
     * mode.<br>
     * name: '<node_name>/escape_reset_theta'.<br>
     * Type and default: double, 1.5708 (PI/2).
     */    
    double escape_reset_theta;

    /**
     * Explanation: The number of samples to use when exploring the x velocity 
     * space.<br>
     * name: '<node_name>/vx_samples'.<br>
     * Type and default: integer, 3.
     */    
    int vx_samples;

    /**
     * Explanation: The number of samples to use when exploring the theta velocity space .<br>
     * name: '<node_name>/vtheta_samples'.<br>
     * Type and default: integer, 20.
     */    
    int vtheta_samples;

    /**
     * Explanation: How far to look ahead in meters when scoring different 
     * in-place-rotation trajectories.<br>
     * name: '<node_name>/heading_lookahead'.<br>
     * Type and default: double, 0.325.
     */    
    double heading_lookahead;
    
    /**
     * Explanation: Determines whether velocity commands are generated for a 
     * holonomic or non-holonomic robot. For holonomic robots, strafing velocity 
     * commands may be issued to the base. For non-holonomic robots, no strafing 
     * velocity commands will be issued.<br>
     * name: '<node_name>/holonomic_robot'.<br>
     * Type and default: bool, true.
     */    
    bool holonomic_robot;

    /**
     * Explanation: Speed used for driving during escapes in meters/sec. Note 
     * that it must be negative in order for the robot to actually reverse. A 
     * positive speed will cause the robot to move forward while attempting to 
     * escape.<br>
     * name: '<node_name>/escape_vel'.<br>
     * Type and default: double, -0.1.
     */    
    double escape_vel;

    /**
     * Explanation: Whether to use the Dynamic Window Approach (DWA)_ or whether
     * to use Trajectory Rollout (NOTE: In our experience DWA worked as well as 
     * Trajectory Rollout and is computationally less expensive. It is possible 
     * that robots with extremely poor acceleration limits could gain from 
     * running Trajectory Rollout).<br>
     * name: '<node_name>/dwa'.<br>
     * Type and default: bool, true.
     */    
    bool dwa;
    
    /**
     * Explanation: Whether to score based on the robot's heading to the path 
     * or its distance from the path.<br>
     * name: '<node_name>/heading_scoring'.<br>
     * Type and default: bool, false.
     */    
    bool heading_scoring;
    
    /**
     * Explanation: How far to look ahead in time in seconds along the simulated
     * trajectory when using heading scoring.<br>
     * name: '<node_name>/heading_scoring_timestep'.<br>
     * Type and default: double, 0.8.
     */    
    double heading_scoring_timestep;

    /**
     * Explanation: Set this to true to allow simple attraction to a goal point 
     * instead of intelligent cost propagation.<br>
     * name: '<node_name>/planner_frequency'.<br>
     * Type and default: bool, false.
     */    
    bool simple_attractor;

    /**
     * Explanation: The step size, in radians, to take between angular samples 
     * on a given trajectory.<br>
     * name: '<node_name>/angular_sim_granularity'.<br>
     * Type and default: double, 0.025.
     */    
    double angular_sim_granularity;
    
    /**
     * Explanation: The strafing velocities that a holonomic robot will consider 
     * in meters/sec.<br>
     * name: '<node_name>/y_vels'.<br>
     * Type and default: list, '[-0.3, -0.1, 0.1, 0.3]'.
     */    
    std::string y_vels;

    /**
     * Explanation: Reset all values at the target dynamic_reconfiguration
     * server to its defaults. ROS topic name: '<node_name>/restore_defaults'.
     * Type and default: bool, false
     */  
    bool restore_defaults;
};

}

#endif /*ROSBASELOCALPLANNERCONFIGURATION_H_*/