/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

/*
 * ROSCostmapConfiguration.cpp
 *
 *  Created on: May 31, 2012
 *      Author: pdressel
 */

#include "ROSCostmapConfiguration.h"

using namespace std;
using namespace airobots;
using namespace xmltio;

namespace airobots {
    
    const string ROSCostmapConfiguration::COSTMAP_CONFIGURATION_TAG = "COSTMAPCONFIGURATION";
    
    const string ROSCostmapConfiguration::TRANSFORM_TOLERANCE_ATTRIBUTE = "transform_tolerance"; 
    const string ROSCostmapConfiguration::UPDATE_FREQUENCY_ATTRIBUTE = "update_frequency";
    const string ROSCostmapConfiguration::PUBLISH_FREQUENCY_ATTRIBUTE = "publish_frequency";
    const string ROSCostmapConfiguration::MAX_OBSTACLE_HEIGHT_ATTRIBUTE = "max_obstacle_height";
    const string ROSCostmapConfiguration::MAX_OBSTACLE_RANGE_ATTRIBUTE = "max_obstacle_range";
    const string ROSCostmapConfiguration::RAYTRACE_RANGE_ATTRIBUTE = "raytrace_range";
    const string ROSCostmapConfiguration::COST_SCALING_FACTOR_ATTRIBUTE = "cost_scaling_factor";
    const string ROSCostmapConfiguration::INFLATION_RADIUS_ATTRIBUTE = "inflation_radius";
    const string ROSCostmapConfiguration::FOOTPRINT_ATTRIBUTE = "footprint";
    const string ROSCostmapConfiguration::ROBOT_RADIUS_ATTRIBUTE = "robot_radius";
    const string ROSCostmapConfiguration::STATIC_MAP_ATTRIBUTE = "static_map";
    const string ROSCostmapConfiguration::ROLLING_WINDOW_ATTRIBUTE = "rolling_window";
    const string ROSCostmapConfiguration::UNKNOWN_COST_VALUE_ATTRIBUTE = "unknown_cost_value";
    const string ROSCostmapConfiguration::WIDTH_ATTRIBUTE = "width";
    const string ROSCostmapConfiguration::HEIGHT_ATTRIBUTE = "height";
    const string ROSCostmapConfiguration::RESOLUTION_ATTRIBUTE = "resolution";
    const string ROSCostmapConfiguration::ORIGIN_X_ATTRIBUTE = "origin_x";
    const string ROSCostmapConfiguration::ORIGIN_Y_ATTRIBUTE = "origin_y";
    const string ROSCostmapConfiguration::PUBLISH_VOXEL_MAP_ATTRIBUTE = "publish_voxel_map";
    const string ROSCostmapConfiguration::LETHAL_COST_THRESHOLD_ATTRIBUTE = "lethal_cost_threshold";
    const string ROSCostmapConfiguration::MAP_TOPIC_ATTRIBUTE = "map_topic";
    const string ROSCostmapConfiguration::MAP_TYPE_ATTRIBUTE = "map_type";
    const string ROSCostmapConfiguration::Z_ORIGIN_ATTRIBUTE = "origin_z";
    const string ROSCostmapConfiguration::Z_RESOLUTION_ATTRIBUTE = "z_resolution";
    const string ROSCostmapConfiguration::Z_VOXELS_ATTRIBUTE = "z_voxels";
    const string ROSCostmapConfiguration::UNKNOWN_THRESHOLD_ATTRIBUTE = "unknown_threshold";
    const string ROSCostmapConfiguration::MARK_THRESHOLD_ATTRIBUTE = "mark_threshold";
    const string ROSCostmapConfiguration::TRACK_UNKNOWN_SPACE_ATTRIBUTE = "track_unknown_space";
    const string ROSCostmapConfiguration::RESTORE_DEFAULTS_ATTRIBUTE = "restore_defaults";
    
    const string ROSCostmapConfiguration::IS_GLOBAL_COSTMAP_ATTRIBUTE = "is_global_costmap";
    
    ROSCostmapConfiguration::ROSCostmapConfiguration(const bool isGlobalCostmap)
    {
        this->isGlobalCostmap = isGlobalCostmap;
        
        this->transformTolerance = 0.0;
        this->updateFrequency = 0.0;
        this->publishFrequency = 0.0;
        this->maxObstacleHeight = 0.0;
        this->maxObstacleRange = 0.0;
        this->raytraceRange = 0.0;
        this->costScalingFactor = 0.0;
        this->inflationRadius = 0.0;
        this->footprint = "";
        this->robotRadius = 0.0;
        this->staticMap = false;
        this->rollingWindow = false;
        this->unknownCostValue = 0;
        this->width = 0;
        this->height = 0;
        this->resolution = 0.0;
        this->originX = 0.0;
        this->originY = 0.0;
        this->publishVoxelMap = false;
        this->lethalCostThreshold = 0;
        this->mapTopic = "";
        this->mapType = "";
        this->zOrigin = 0.0;
        this->zResolution = 0.0;
        this->zVoxels = 0;
        this->unknownThreshold = 0;
        this->markThreshold = 0;        
        this->trackUnknownSpace = false;
        this->restoreDefaults = false;
    }
    
    string ROSCostmapConfiguration::getBaseTag()
    {
        return ROSDynamicConfiguration::getBaseTag();
    }
    
    string ROSCostmapConfiguration::getTemplate()
    {
    	return "<" + getBaseTag() + ">"		
		+ airobots::Timestamp::getTemplate()
                + "<GENERATOR/>"
		+ "<" + COSTMAP_CONFIGURATION_TAG + " " + 
                        + "transform_tolerance=\"\" "
                        + "update_frequency=\"\" " 
                        + "publish_frequency=\"\" "
                        + "max_obstacle_height=\"\" " 
                        + "max_obstacle_range=\"\" "
                        + "raytrace_range=\"\" " 
                        + "cost_scaling_factor=\"\" "
                        + "inflation_radius=\"\" " 
                        + "footprint=\"\" "
                        + "robot_radius=\"\" " 
                        + "static_map=\"\" "
                        + "rolling_window=\"\" " 
                        + "unknown_cost_value=\"\" "
                        + "width=\"\" " 
                        + "height=\"\" "
                        + "resolution=\"\" " 
                        + "origin_x=\"\" "
                        + "origin_y=\"\" " 
                        + "lethal_cost_threshold=\"\" "    
                        + "map_topic=\"\" "
                        + "map_type=\"\" "  
			+ "publish_voxel_map=\"\" "
                        + "origin_z=\"\" "
                        + "z_resolution=\"\" "  
                        + "z_voxels=\"\" "
                        + "unknown_threshold=\"\" "  
                        + "mark_threshold=\"\" "
                        + "track_unknown_space=\"\" "  
                        + "restore_defaults=\"\" "
                        + "is_global_costmap=\"\" />"  
		+ "</" + getBaseTag() + ">";
    }
    
    void ROSCostmapConfiguration::sanitizeLocation(xmltio::Location &loc) const
    {
        ROSDynamicConfiguration::sanitizeLocation(loc);
    }
    
    void ROSCostmapConfiguration::fillInto(xmltio::Location &loc) const
    {
        string prefix = COSTMAP_CONFIGURATION_TAG 
                + ROSDynamicConfiguration::X_PATH_SEPARATOR 
                + ROSDynamicConfiguration::ATTRIBUTE_PREFIX;
	
        
        Type::fillInto(loc);
        
        loc[XPath(prefix + TRANSFORM_TOLERANCE_ATTRIBUTE)]      = this->transformTolerance;
	loc[XPath(prefix + UPDATE_FREQUENCY_ATTRIBUTE)]         = this->updateFrequency;
        loc[XPath(prefix + PUBLISH_FREQUENCY_ATTRIBUTE)]        = this->publishFrequency;
        loc[XPath(prefix + MAX_OBSTACLE_HEIGHT_ATTRIBUTE)]      = this->maxObstacleHeight;
        loc[XPath(prefix + MAX_OBSTACLE_RANGE_ATTRIBUTE)]       = this->maxObstacleRange;
        loc[XPath(prefix + RAYTRACE_RANGE_ATTRIBUTE)]           = this->raytraceRange;
        loc[XPath(prefix + COST_SCALING_FACTOR_ATTRIBUTE)]      = this->costScalingFactor;
        loc[XPath(prefix + INFLATION_RADIUS_ATTRIBUTE)]         = this->inflationRadius;
        loc[XPath(prefix + FOOTPRINT_ATTRIBUTE)]                = this->footprint;
        loc[XPath(prefix + ROBOT_RADIUS_ATTRIBUTE)]             = this->robotRadius;
        loc[XPath(prefix + STATIC_MAP_ATTRIBUTE)]               = this->staticMap == 1 ? ROSDynamicConfiguration::TRUE_STRING : ROSDynamicConfiguration::FALSE_STRING;
        loc[XPath(prefix + ROLLING_WINDOW_ATTRIBUTE)]           = this->rollingWindow == 1 ? ROSDynamicConfiguration::TRUE_STRING : ROSDynamicConfiguration::FALSE_STRING;
        loc[XPath(prefix + UNKNOWN_COST_VALUE_ATTRIBUTE)]       = this->unknownCostValue;
        loc[XPath(prefix + WIDTH_ATTRIBUTE)]                    = this->width;
        loc[XPath(prefix + HEIGHT_ATTRIBUTE)]                   = this->height;
        loc[XPath(prefix + RESOLUTION_ATTRIBUTE)]               = this->resolution;
        loc[XPath(prefix + ORIGIN_X_ATTRIBUTE)]                 = this->originX;
        loc[XPath(prefix + ORIGIN_Y_ATTRIBUTE)]                 = this->originY;
        loc[XPath(prefix + PUBLISH_VOXEL_MAP_ATTRIBUTE)]        = this->publishVoxelMap == 1 ? ROSDynamicConfiguration::TRUE_STRING : ROSDynamicConfiguration::FALSE_STRING;
        loc[XPath(prefix + LETHAL_COST_THRESHOLD_ATTRIBUTE)]    = this->lethalCostThreshold;
        loc[XPath(prefix + MAP_TOPIC_ATTRIBUTE)]                = this->mapTopic;
        loc[XPath(prefix + MAP_TYPE_ATTRIBUTE)]                 = this->mapType;
        loc[XPath(prefix + Z_ORIGIN_ATTRIBUTE)]                 = this->zOrigin;
        loc[XPath(prefix + Z_RESOLUTION_ATTRIBUTE)]             = this->zResolution;
        loc[XPath(prefix + Z_VOXELS_ATTRIBUTE)]                 = this->zVoxels;
        loc[XPath(prefix + UNKNOWN_THRESHOLD_ATTRIBUTE)]        = this->unknownThreshold;
        loc[XPath(prefix + MARK_THRESHOLD_ATTRIBUTE)]           = this->markThreshold;        
        loc[XPath(prefix + TRACK_UNKNOWN_SPACE_ATTRIBUTE)]      = this->trackUnknownSpace == 1 ? ROSDynamicConfiguration::TRUE_STRING : ROSDynamicConfiguration::FALSE_STRING;
        loc[XPath(prefix + RESTORE_DEFAULTS_ATTRIBUTE)]         = this->restoreDefaults == 1 ? ROSDynamicConfiguration::TRUE_STRING : ROSDynamicConfiguration::FALSE_STRING;
        loc[XPath(prefix + IS_GLOBAL_COSTMAP_ATTRIBUTE)]        = this->isGlobalCostmap == 1 ? ROSDynamicConfiguration::TRUE_STRING : ROSDynamicConfiguration::FALSE_STRING; 
    }
    
    void ROSCostmapConfiguration::fromLocation(const xmltio::Location &loc, ROSCostmapConfiguration &config)
    {
        string prefix = COSTMAP_CONFIGURATION_TAG 
                + ROSDynamicConfiguration::X_PATH_SEPARATOR 
                + ROSDynamicConfiguration::ATTRIBUTE_PREFIX;
	
        
        Type::fromLocation(loc, config);
        
        config.setTransformTolerance(   extract<double>(loc[XPath(prefix + TRANSFORM_TOLERANCE_ATTRIBUTE)]));
	config.setUpdateFrequency(      extract<double>(loc[XPath(prefix + UPDATE_FREQUENCY_ATTRIBUTE)]));
        config.setPublishFrequency(     extract<double>(loc[XPath(prefix + PUBLISH_FREQUENCY_ATTRIBUTE)]));
        config.setMaxObstacleHeight(    extract<double>(loc[XPath(prefix + MAX_OBSTACLE_HEIGHT_ATTRIBUTE)]));
        config.setMaxObstacleRange(     extract<double>(loc[XPath(prefix + MAX_OBSTACLE_RANGE_ATTRIBUTE)]));
        config.setRaytraceRange(        extract<double>(loc[XPath(prefix + RAYTRACE_RANGE_ATTRIBUTE)]));
        config.setCostScalingFactor(    extract<double>(loc[XPath(prefix + COST_SCALING_FACTOR_ATTRIBUTE)]));
        config.setInflationRadius(      extract<double>(loc[XPath(prefix + INFLATION_RADIUS_ATTRIBUTE)]));
        config.setFootprint(            extract<string>(loc[XPath(prefix + FOOTPRINT_ATTRIBUTE)]));   
        config.setRobotRadius(          extract<double>(loc[XPath(prefix + ROBOT_RADIUS_ATTRIBUTE)]));
        
        string staticMapString             = extract<string>(loc[XPath(prefix + STATIC_MAP_ATTRIBUTE)]);
        config.setStaticMap(staticMapString.compare(TRUE_STRING) == 0 ? true : false);                
        string rollingWindowString         = extract<string>(loc[XPath(prefix + ROLLING_WINDOW_ATTRIBUTE)]);
        config.setRollingWindow(rollingWindowString.compare(TRUE_STRING) == 0 ? true : false);
                
        config.setUnknownCostValue(     extract<int>(loc[XPath(prefix + UNKNOWN_COST_VALUE_ATTRIBUTE)]));
        config.setWidth(                extract<int>(loc[XPath(prefix + WIDTH_ATTRIBUTE)]));
        config.setHeight(               extract<int>(loc[XPath(prefix + HEIGHT_ATTRIBUTE)]));
        config.setResolution(           extract<double>(loc[XPath(prefix + RESOLUTION_ATTRIBUTE)]));
        config.setOriginX(              extract<double>(loc[XPath(prefix + ORIGIN_X_ATTRIBUTE)]));
        config.setOriginY(              extract<double>(loc[XPath(prefix + ORIGIN_Y_ATTRIBUTE)]));
        
        string publishVoxelMapString       = extract<string>(loc[XPath(prefix + PUBLISH_VOXEL_MAP_ATTRIBUTE)]);
        config.setPublishVoxelMap(publishVoxelMapString.compare(TRUE_STRING) == 0 ? true : false);
        
        config.setLethalCostThreshold(  extract<int>(loc[XPath(prefix + LETHAL_COST_THRESHOLD_ATTRIBUTE)]));
        config.setMapTopic(             extract<string>(loc[XPath(prefix + MAP_TOPIC_ATTRIBUTE)]));
        config.setMapType(              extract<string>(loc[XPath(prefix + MAP_TYPE_ATTRIBUTE)]));
        config.setzOrigin(              extract<double>(loc[XPath(prefix + Z_ORIGIN_ATTRIBUTE)]));
        config.setzResolution(          extract<double>(loc[XPath(prefix + Z_RESOLUTION_ATTRIBUTE)]));
        config.setzVoxels(              extract<int>(loc[XPath(prefix + Z_VOXELS_ATTRIBUTE)]));
        config.setUnknownThreshold(     extract<int>(loc[XPath(prefix + UNKNOWN_THRESHOLD_ATTRIBUTE)]));
        config.setMarkThreshold(        extract<int>(loc[XPath(prefix + MARK_THRESHOLD_ATTRIBUTE)]));
        
        string trackUnknownSpaceString     = extract<string>(loc[XPath(prefix + TRACK_UNKNOWN_SPACE_ATTRIBUTE)]);
        config.setTrackUnknownSpace(trackUnknownSpaceString.compare(TRUE_STRING) == 0 ? true : false);        
        string restoreDefaultsString       = extract<string>(loc[XPath(prefix + RESTORE_DEFAULTS_ATTRIBUTE)]);
        config.setRestoreDefaults(restoreDefaultsString.compare(TRUE_STRING) == 0 ? true : false);        
        string isGlobalCostmapString       = extract<string>(loc[XPath(prefix + IS_GLOBAL_COSTMAP_ATTRIBUTE)]);
        config.setIsGlobalCostmap(isGlobalCostmapString.compare(TRUE_STRING) == 0 ? true : false);
    }
}