/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

 /** ==========================================================================
 * 
 * @file		PersonDataList.h --
 * @brief		Header file for the list of person data.
 *
 * @author	Denis Schulze <dschulze@techfak.uni-bielefeld.de>
 * 
 * ========================================================================== 
 **/

#ifndef __PERSONDATALIST_H__
#define __PERSONDATALIST_H__

#include <vector>

#include "../../Type.h"
#include "PersonData.h"

namespace airobots {

class PersonDataList : public Type {

	public:

		/** Constructor. */
		PersonDataList();

		/** Deconstructor. */
		~PersonDataList();

		static std::string getBaseTag();
		
		static std::string getTemplate();

		void addPersonData(airobots::PersonData data);

		std::vector<PersonData> getPersonData();

		int size();

		void sanitizeLocation(xmltio::Location &loc) const;
		void fillInto(xmltio::Location &loc) const;
		static void fromLocation(const xmltio::Location &loc, PersonDataList &list);
	
	private:

		/** A list of all stored person data in this list. */
		std::vector<PersonData> personData;		

};

}

#endif /* __PERSONDATALIST_H__ */
