/*
 * PointCloudGrasping.cpp
 *
 *  Created on: Jun 14, 2011
 *      Author: plueckin
 */

#include "PointCloudGrasping.h"

using namespace airobots;
using namespace xmltio;
using namespace std;

static const string GRASPPOINT = "GRASPING_POINT";

PointCloudGrasping::PointCloudGrasping() :
		Type() {
	iLU = MILLIMETER;
}

PointCloudGrasping::PointCloudGrasping(std::string generator) :
		Type(generator) {
	iLU = MILLIMETER;
}

PointCloudGrasping::PointCloudGrasping(const PointCloudGrasping &other) {
	iLU = other.iLU;
	convex_hull = other.convex_hull;
	grasppnt = other.grasppnt;
	label = other.label;
	height = other.height;
	width = other.width;
	depth = other.depth;
}

PointCloudGrasping::PointCloudGrasping(PointCloud chull, Point3D gpnt,
		string lbl, double h, double w, double d, LengthUnit ilu) :
		convex_hull(chull), grasppnt(gpnt), label(lbl), height(h), width(w), depth(
				d), iLU(ilu) {
}

PointCloudGrasping::~PointCloudGrasping() {
	// TODO Auto-generated destructor stub
}

string PointCloudGrasping::getBaseTag() {
	return "POINTCLOUDGRASPING";
}

string PointCloudGrasping::getTemplate() {

	string graspingpoint = Point3D::getTemplate();
	uint pos = graspingpoint.find(Point3D::getBaseTag());
	while (pos != graspingpoint.npos) {
		graspingpoint.replace(pos, Point3D::getBaseTag().length(), GRASPPOINT);
		pos = graspingpoint.find(Point3D::getBaseTag());
	}

	return std::string("<") + getBaseTag() + "><GENERATOR/>"
			+ Timestamp::getTemplate() + graspingpoint + "<LABEL value=\"\" />"
			+ +"<HEIGHT valh=\"\" />" + "<WIDTH valw=\"\" />"
			+ "<DEPTH vald=\"\" />" + "</" + getBaseTag() + ">";
}

void PointCloudGrasping::fillInto(Location &loc) const {
	Type::fillInto(loc);

	loc[XPath(GRASPPOINT)] = grasppnt;
	loc[XPath("LABEL/@value")] = label;
	loc[XPath("HEIGHT/@valh")] = height;
	loc[XPath("WIDTH/@valw")] = width;
	loc[XPath("DEPTH/@vald")] = depth;

	Location loc_hull(PointCloud::getTemplate(),
			XPath(PointCloud::getBaseTag()));
	loc_hull = convex_hull;

	loc.add(loc_hull);
}

void PointCloudGrasping::fromLocation(const Location &loc,
		PointCloudGrasping &pcg) {

	Type::fromLocation(loc, pcg);

	pcg.convex_hull = extract<PointCloud>(loc[XPath(PointCloud::getBaseTag())]);
	pcg.grasppnt = extract<Point3D>(loc[XPath(GRASPPOINT)]);

	double h = extract<double>(loc[XPath("HEIGHT/@valh")]);
	double w = extract<double>(loc[XPath("WIDTH/@valw")]);
	double d = extract<double>(loc[XPath("DEPTH/@vald")]);

	pcg.label = extract<string>(loc[XPath("LABEL/@value")]);
	pcg.height = h;
	pcg.width = w;
	pcg.depth = d;
}

void PointCloudGrasping::sanitizeLocation(Location &loc) const {
	XPath xp(getBaseTag());
	while (XPathIterator i = xp.evaluate(loc)) {
		Location l(*i);
		loc.remove(l);
	}
}

bool PointCloudGrasping::operator==(const PointCloudGrasping &other) const {
	if ((other.height != height) || (other.width != width)
			|| (other.depth != depth))
		return false;
	if (!(other.grasppnt == grasppnt))
		return false;
	if (!(other.label == label))
		return false;
	if (!(other.convex_hull == convex_hull))
		return false;
	return true;
}

void PointCloudGrasping::setHull(PointCloud pc) {
	this->convex_hull = pc;
}

void PointCloudGrasping::setGraspPnt(Point3D gpnt) {
	this->grasppnt = gpnt;
}

void PointCloudGrasping::setLabel(string label) {
	this->label = label;
}

void PointCloudGrasping::setHeight(double h, LengthUnit lU) {
	this->height = UnitConverter::convert(h, lU, iLU);
}

void PointCloudGrasping::setWidth(double w, LengthUnit lU) {
	this->width = UnitConverter::convert(w, lU, iLU);
	;
}

void PointCloudGrasping::setDepth(double d, LengthUnit lU) {
	this->depth = UnitConverter::convert(d, lU, iLU);
	;
}

PointCloud PointCloudGrasping::getHull() {
	return convex_hull;
}

Point3D PointCloudGrasping::getGraspPnt() {
	return grasppnt;
}

std::string PointCloudGrasping::getLabel() {
	return label;
}

double PointCloudGrasping::getHeight(LengthUnit lU) {
	return UnitConverter::convert(height, iLU, lU);
}

double PointCloudGrasping::getWidth(LengthUnit lU) {
	return UnitConverter::convert(width, iLU, lU);
}

double PointCloudGrasping::getDepth(LengthUnit lU) {
	return UnitConverter::convert(depth, iLU, lU);
}

