/*
 * PointCloudGrasping.h
 *
 *  Created on: Jun 14, 2011
 *      Author: plueckin
 */

#ifndef POINTCLOUDGRASPING_H_
#define POINTCLOUDGRASPING_H_

#include "../../Type.h"
#include "../geometry/Point3D.h"
#include "PointCloud.h"

namespace airobots {

class PointCloudGrasping: public Type {
public:
	PointCloudGrasping();
	PointCloudGrasping(std:: string generator);
	PointCloudGrasping(const PointCloudGrasping &pcg);
	PointCloudGrasping(PointCloud pc, Point3D gpnt, std::string lbl, double h,
			double w, double d, LengthUnit ilu);
	virtual ~PointCloudGrasping();

	static std::string getBaseTag();
	static std::string getTemplate();

	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc,
			PointCloudGrasping &type);
	void sanitizeLocation(xmltio::Location &loc) const;

	PointCloud getHull();
	Point3D getGraspPnt();
	std::string getLabel();
	double getHeight(LengthUnit lU);
	double getWidth(LengthUnit lU);
	double getDepth(LengthUnit lU);

	void setHull(PointCloud pc);
	void setGraspPnt(Point3D gpnt);
	void setLabel(std::string lbl);
	void setHeight(double h,LengthUnit lU);
	void setWidth(double w,LengthUnit lU);
	void setDepth(double d,LengthUnit lU);

	bool operator==(const PointCloudGrasping &other) const;

private:

	PointCloud convex_hull;
	Point3D grasppnt;
	std::string label;

	double height;
	double width;
	double depth;

	LengthUnit iLU;

};

}

#endif /* POINTCLOUDGRASPING_H_ */
