/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Robin Schiewer, Sebastian 
 * Meyer zu Borgsen, Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Tristan 
 * Walter, Viktor Losing, Viktor Richter
 */
package de.unibi.airobots.btl.data.person;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.ElementParser;
import de.unibi.airobots.btl.units.*;

public class Shirt extends Type implements Percept {
	
	private static AngleUnit iAU = AngleUnit.RADIAN;
	private static LengthUnit iLU = LengthUnit.MILLIMETER;

	@Override
	public String getPerceptName() {
		return "shirt";
	}

	public static String getBaseTag() {
		return "SHIRT";
	}

	public static void fromElement(Element element, Shirt type) {

		Type.fromElement(element, type);

		Nodes nodes = element.query("*");
		for (int i = 0; i < nodes.size(); i++) {
			Node node = nodes.get(i);
			if (node instanceof Element) {
				Element elem = (Element) node;
				if (elem.getLocalName().equals("POSITION")) {
					type.setAngle((float) ElementParser
							.getDoubleAttributeValue(elem, "angle"), iAU);
					type.setAngleVariance((float) ElementParser
							.getDoubleAttributeValue(elem, "angleVariance"), iAU);
					type.setDistance((float) ElementParser
							.getDoubleAttributeValue(elem, "distance"), iLU);
					type.setDistanceVariance((float) ElementParser
							.getDoubleAttributeValue(elem, "distanceVariance"), iLU);
				}
			}
		}

	}

	public void fillInto(Element parent) {

		super.fillInto(parent);

		Element posElement = new Element("POSITION");
		parent.appendChild(posElement);
		posElement.addAttribute(new Attribute("" + "angle", String
				.valueOf(getAngle(iAU))));
		posElement.addAttribute(new Attribute("" + "angleVariance", String
				.valueOf(getAngleVariance(iAU))));
		posElement.addAttribute(new Attribute("" + "distance", String
				.valueOf(getDistance(iLU))));
		posElement.addAttribute(new Attribute("" + "distanceVariance", String
				.valueOf(getDistanceVariance(iLU))));

	}

	public void setAngle(float angle, AngleUnit u) {
		this.angle = UnitConverter.convert(angle, u, iAU);
	}

	public float getAngle(AngleUnit u) {
		return UnitConverter.convert(angle, iAU, u);
	}

	public void setAngleVariance(float angleVariance, AngleUnit u) {
		this.angleVariance = UnitConverter.convert(angleVariance, u, iAU);
	}

	public float getAngleVariance(AngleUnit u) {
		return UnitConverter.convert(angleVariance, iAU, u);
	}

	public void setDistance(float distance, LengthUnit u) {
		this.distance = UnitConverter.convert(distance, u, iLU);
	}

	public float getDistance(LengthUnit u) {
		return UnitConverter.convert(distance, iLU, u);
	}

	public void setDistanceVariance(float distanceVariance, LengthUnit u) {
		this.distanceVariance = UnitConverter.convert(distanceVariance, u, iLU);
	}

	public float getDistanceVariance(LengthUnit u) {
		return UnitConverter.convert(distanceVariance, iLU, u);
	}

	public boolean equals(Shirt other) {
		return (this.angle == other.angle)
				&& (this.angleVariance == other.angleVariance)
				&& (this.distance == other.distance)
				&& (this.distanceVariance == other.distanceVariance);
	}

	/** The angle to the shirt percept. */
	private float angle;

	/** The variance in the angle value. */
	private float angleVariance;

	/** The distance to the shirt percept. */
	private float distance;

	/** The variance in the distance values. */
	private float distanceVariance;

}
