/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Robin Schiewer, Sebastian 
 * Meyer zu Borgsen, Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Tristan 
 * Walter, Viktor Losing, Viktor Richter
 */
package de.unibi.airobots.btl.data.speechrec;

/**
 * Domain class representing the grammar tree provided by the speech recognizer.
 * The GrammarTree itself is kind of a pseudo start symbol, since the real start
 * symbol might be hidden in the speech recognizers output. Furthermore the
 * GrammarTree can also include partially parsed utterances e.g. if the parser
 * had to cancel and reinitialize to parse the utterance.
 * 
 * @author lschilli
 * 
 */
public class GrammarTree extends GrammarNonTerminal {

	private boolean cancel;
	private boolean fault;
	private boolean skip;

	public GrammarTree() {
		super(null);
	}

	/**
	 * Indicates the parser has at least once canceled (and reinitialized)
	 * during parsing
	 * 
	 * @return
	 */
	public boolean isCancel() {
		return cancel;
	}

	public void setCancel(boolean cancel) {
		this.cancel = cancel;
	}

	/**
	 * Indicates the parser has rated at least one word as a fault (not matching
	 * to the grammar structure).
	 * 
	 * @return
	 */
	public boolean isFault() {
		return fault;
	}

	public void setFault(boolean fault) {
		this.fault = fault;
	}

	/**
	 * Indicates the parser has skipped at least one word (if allowed to).
	 * 
	 * @return
	 */
	public boolean isSkip() {
		return skip;
	}

	public void setSkip(boolean skip) {
		this.skip = skip;
	}

}