/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke, Lukas Kettenbach
 */
package de.unibi.airobots.btl.data.map;

import de.unibi.airobots.btl.data.common.Timestamp;
import de.unibi.airobots.btl.data.navigation.PositionData;
import de.unibi.airobots.btl.units.AngleUnit;
import de.unibi.airobots.btl.units.LengthUnit;
import nu.xom.Element;
import static org.junit.Assert.assertTrue;
import org.junit.Test;

/**
 * Test cases for {@link Viewpoint}.
 *  
 * @author lkettenb
 */
public class ViewpointTest {
    @Test
	public void selfCompatibility() throws Exception {
        PositionData coordinates = new PositionData(0.0, 1.0, 2.0, 
                new Timestamp(), LengthUnit.METER, AngleUnit.RADIAN);
        Viewpoint viewpoint = new Viewpoint(coordinates);
        
        assertTrue("Standard category not working.", 
                viewpoint.getCategory().equals(
                Viewpoint.ViewpointCategory.VIEW));
        
        viewpoint = new Viewpoint(coordinates, 
                Viewpoint.ViewpointCategory.OBJECT, 
                "Bottle");
        
        assertTrue("Label/name not working.", 
                viewpoint.getLabel().equals("Bottle"));
        assertTrue("Category not working.", 
                viewpoint.getCategory().equals(Viewpoint.ViewpointCategory.OBJECT));
        
        Element element = new Element(Viewpoint.getBaseTag());
        viewpoint.fillInto(element);
        System.out.println(element.toXML());
        Viewpoint parsedViewpoint = Viewpoint.fromElement(element);
        
        assertTrue("Parsing label not working.", 
                viewpoint.getLabel().equals(parsedViewpoint.getLabel()));
        assertTrue("Parsing category not working.", 
                viewpoint.getCategory().equals(parsedViewpoint.getCategory()));
        assertTrue("Parsing coordinates not working.", 
                viewpoint.getCoordinates().equals(parsedViewpoint.getCoordinates()));
    }
}
