/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/
package de.unibi.airobots.btl.data.navigation;

import static org.junit.Assert.*;
import nu.xom.Document;
import nu.xom.Element;
import org.apache.log4j.Logger;
import org.junit.Test;

/**
 * Test cases for {@link ROSMoveBaseConfigurationTest}
 * 
 * @author pdressel
 */
public class ROSMoveBaseConfigurationTest {
    
    private final static Double DELTA = 0.0001;
    private Logger logger = Logger.getLogger(getClass());
        
    public static ROSMoveBaseConfiguration createMockObject() {
        
        ROSMoveBaseConfiguration moveBaseConfiguration = new ROSMoveBaseConfiguration();
        moveBaseConfiguration.setBaseGlobalPlanner("navfn/NavfnROS");
        moveBaseConfiguration.setBaseLocalPlanner("base_local_planner/TrajectoryPlannerROS");
        moveBaseConfiguration.setClearingRotationAllowed(Math.random() > Math.random());
        moveBaseConfiguration.setConservativeResetDist(Math.random());
        moveBaseConfiguration.setControllerFrequency(Math.random());
        moveBaseConfiguration.setControllerPatience(Math.random());
        moveBaseConfiguration.setOscillationDistance(Math.random());
        moveBaseConfiguration.setOscillationTimeout(Math.random());
        moveBaseConfiguration.setPlannerFrequency(Math.random());
        moveBaseConfiguration.setPlannerPatience(Math.random());
        moveBaseConfiguration.setRecoveryBehaviorEnabled(Math.random() > Math.random());
        moveBaseConfiguration.setRestoreDefaults(Math.random() > Math.random());
        moveBaseConfiguration.setShutdownCostmaps(Math.random() > Math.random());
        
        return moveBaseConfiguration;
    }    
    
    @Test
    public void selfCompatibility() throws Exception {
        
        ROSMoveBaseConfiguration original = createMockObject();
        ROSMoveBaseConfiguration parsed = new ROSMoveBaseConfiguration();
        
        Document doc = new Document(new Element(ROSMoveBaseConfiguration.getBaseTag()));
        original.fillInto(doc.getRootElement());
        logger.info(doc.toXML());
        
        ROSMoveBaseConfiguration.fromElement(doc.getRootElement(), parsed);
        
        assertEquals(original.getBaseGlobalPlanner(), parsed.getBaseGlobalPlanner());
        assertEquals(original.getBaseLocalPlanner(), parsed.getBaseLocalPlanner());
        assertEquals(original.isClearingRotationAllowed(), parsed.isClearingRotationAllowed());
        assertEquals(original.getConservativeResetDist(), parsed.getConservativeResetDist(), DELTA);
        assertEquals(original.getControllerFrequency(), parsed.getControllerFrequency(), DELTA);
        assertEquals(original.getControllerPatience(), parsed.getControllerPatience(), DELTA);
        assertEquals(original.getOscillationDistance(), parsed.getOscillationDistance(), DELTA);
        assertEquals(original.getOscillationTimeout(), parsed.getOscillationTimeout(), DELTA);
        assertEquals(original.getPlannerFrequency(), parsed.getPlannerFrequency(), DELTA);
        assertEquals(original.getPlannerPatience(), parsed.getPlannerPatience(), DELTA);
        assertEquals(original.isRecoveryBehaviorEnabled(), parsed.isRecoveryBehaviorEnabled());
        assertEquals(original.isRestoreDefaults(), parsed.isRestoreDefaults());
        assertEquals(original.isShutdownCostmaps(), parsed.isShutdownCostmaps());
    }    
}
