/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/
package de.unibi.airobots.btl.unit;

import static org.junit.Assert.*;

import org.junit.Test;

import de.unibi.airobots.btl.units.AngleUnit;
import de.unibi.airobots.btl.units.LengthUnit;
import de.unibi.airobots.btl.units.RotationalSpeedUnit;
import de.unibi.airobots.btl.units.SpeedUnit;
import de.unibi.airobots.btl.units.TimeUnit;
import de.unibi.airobots.btl.units.UnitConverter;

/**
 * Test cases for {@link de.unibi.airobots.btl.Type}.
 * 
 * @author lziegler
 */
public class UnitConverterTest {

	private static final float DELTA = 0.001f;

	@Test
	public void testConvert() throws Exception {
		
		// ############# LENGTH ###############

		// INTEGER
		assertEquals(1, UnitConverter.convert(1, LengthUnit.MILLIMETER, LengthUnit.MILLIMETER));
		assertEquals(0, UnitConverter.convert(1, LengthUnit.MILLIMETER, LengthUnit.CENTIMETER));
		assertEquals(0, UnitConverter.convert(1, LengthUnit.MILLIMETER, LengthUnit.METER));
		
		assertEquals(1000, UnitConverter.convert(1000, LengthUnit.MILLIMETER, LengthUnit.MILLIMETER));
		assertEquals(100, UnitConverter.convert(1000, LengthUnit.MILLIMETER, LengthUnit.CENTIMETER));
		assertEquals(1, UnitConverter.convert(1000, LengthUnit.MILLIMETER, LengthUnit.METER));
		
		assertEquals(10000, UnitConverter.convert(1000, LengthUnit.CENTIMETER, LengthUnit.MILLIMETER));
		assertEquals(1000, UnitConverter.convert(1000, LengthUnit.CENTIMETER, LengthUnit.CENTIMETER));
		assertEquals(10, UnitConverter.convert(1000, LengthUnit.CENTIMETER, LengthUnit.METER));
		
		assertEquals(1000000, UnitConverter.convert(1000, LengthUnit.METER, LengthUnit.MILLIMETER));
		assertEquals(100000, UnitConverter.convert(1000, LengthUnit.METER, LengthUnit.CENTIMETER));
		assertEquals(1000, UnitConverter.convert(1000, LengthUnit.METER, LengthUnit.METER));
		
		// LONG
		assertEquals(1l, UnitConverter.convert(1l, LengthUnit.MILLIMETER, LengthUnit.MILLIMETER));
		assertEquals(0l, UnitConverter.convert(1l, LengthUnit.MILLIMETER, LengthUnit.CENTIMETER));
		assertEquals(0l, UnitConverter.convert(1l, LengthUnit.MILLIMETER, LengthUnit.METER));

		assertEquals(1000l, UnitConverter.convert(1000l, LengthUnit.MILLIMETER, LengthUnit.MILLIMETER));
		assertEquals(100l, UnitConverter.convert(1000l, LengthUnit.MILLIMETER, LengthUnit.CENTIMETER));
		assertEquals(1l, UnitConverter.convert(1000l, LengthUnit.MILLIMETER, LengthUnit.METER));
		
		assertEquals(10000l, UnitConverter.convert(1000l, LengthUnit.CENTIMETER, LengthUnit.MILLIMETER));
		assertEquals(1000l, UnitConverter.convert(1000l, LengthUnit.CENTIMETER, LengthUnit.CENTIMETER));
		assertEquals(10l, UnitConverter.convert(1000l, LengthUnit.CENTIMETER, LengthUnit.METER));
		
		assertEquals(1000000l, UnitConverter.convert(1000l, LengthUnit.METER, LengthUnit.MILLIMETER));
		assertEquals(100000l, UnitConverter.convert(1000l, LengthUnit.METER, LengthUnit.CENTIMETER));
		assertEquals(1000l, UnitConverter.convert(1000l, LengthUnit.METER, LengthUnit.METER));
		
		// FLOAT
		assertEquals(1f, UnitConverter.convert(1f, LengthUnit.MILLIMETER, LengthUnit.MILLIMETER), DELTA);
		assertEquals(0.1f, UnitConverter.convert(1f, LengthUnit.MILLIMETER, LengthUnit.CENTIMETER), DELTA);
		assertEquals(0.001f, UnitConverter.convert(1f, LengthUnit.MILLIMETER, LengthUnit.METER), DELTA);

		assertEquals(1000f, UnitConverter.convert(1000f, LengthUnit.MILLIMETER, LengthUnit.MILLIMETER), DELTA);
		assertEquals(100f, UnitConverter.convert(1000f, LengthUnit.MILLIMETER, LengthUnit.CENTIMETER), DELTA);
		assertEquals(1f, UnitConverter.convert(1000f, LengthUnit.MILLIMETER, LengthUnit.METER), DELTA);
		
		assertEquals(10000f, UnitConverter.convert(1000f, LengthUnit.CENTIMETER, LengthUnit.MILLIMETER), DELTA);
		assertEquals(1000f, UnitConverter.convert(1000f, LengthUnit.CENTIMETER, LengthUnit.CENTIMETER), DELTA);
		assertEquals(10f, UnitConverter.convert(1000f, LengthUnit.CENTIMETER, LengthUnit.METER), DELTA);
		
		assertEquals(1000000f, UnitConverter.convert(1000f, LengthUnit.METER, LengthUnit.MILLIMETER), DELTA);
		assertEquals(100000f, UnitConverter.convert(1000f, LengthUnit.METER, LengthUnit.CENTIMETER), DELTA);
		assertEquals(1000f, UnitConverter.convert(1000f, LengthUnit.METER, LengthUnit.METER), DELTA);
		
		// DOUBLE
		assertEquals(1d, UnitConverter.convert(1d, LengthUnit.MILLIMETER, LengthUnit.MILLIMETER), DELTA);
		assertEquals(0.1d, UnitConverter.convert(1d, LengthUnit.MILLIMETER, LengthUnit.CENTIMETER), DELTA);
		assertEquals(0.001d, UnitConverter.convert(1d, LengthUnit.MILLIMETER, LengthUnit.METER), DELTA);

		assertEquals(1000d, UnitConverter.convert(1000d, LengthUnit.MILLIMETER, LengthUnit.MILLIMETER), DELTA);
		assertEquals(100d, UnitConverter.convert(1000d, LengthUnit.MILLIMETER, LengthUnit.CENTIMETER), DELTA);
		assertEquals(1d, UnitConverter.convert(1000d, LengthUnit.MILLIMETER, LengthUnit.METER), DELTA);
		
		assertEquals(10000d, UnitConverter.convert(1000d, LengthUnit.CENTIMETER, LengthUnit.MILLIMETER), DELTA);
		assertEquals(1000d, UnitConverter.convert(1000d, LengthUnit.CENTIMETER, LengthUnit.CENTIMETER), DELTA);
		assertEquals(10d, UnitConverter.convert(1000d, LengthUnit.CENTIMETER, LengthUnit.METER), DELTA);
		
		assertEquals(1000000d, UnitConverter.convert(1000d, LengthUnit.METER, LengthUnit.MILLIMETER), DELTA);
		assertEquals(100000d, UnitConverter.convert(1000d, LengthUnit.METER, LengthUnit.CENTIMETER), DELTA);
		assertEquals(1000d, UnitConverter.convert(1000d, LengthUnit.METER, LengthUnit.METER), DELTA);
		
		
		// ############# ANGLE ###############
		
		// FLOAT
		assertEquals((float) Math.PI, UnitConverter.convert(180f, AngleUnit.DEGREE, AngleUnit.RADIAN), DELTA);
		assertEquals((float) Math.PI/2f, UnitConverter.convert(90f, AngleUnit.DEGREE, AngleUnit.RADIAN), DELTA);
		assertEquals((float) -Math.PI/2f, UnitConverter.convert(-90f, AngleUnit.DEGREE, AngleUnit.RADIAN), DELTA);
		assertEquals(0f, UnitConverter.convert(0f, AngleUnit.DEGREE, AngleUnit.RADIAN), DELTA);
		
		assertEquals(180f, UnitConverter.convert((float) Math.PI, AngleUnit.RADIAN, AngleUnit.DEGREE), DELTA);
		assertEquals(90f, UnitConverter.convert((float) Math.PI/2f, AngleUnit.RADIAN, AngleUnit.DEGREE), DELTA);
		assertEquals(-90f, UnitConverter.convert((float) -Math.PI/2f, AngleUnit.RADIAN, AngleUnit.DEGREE), DELTA);
		assertEquals(0f, UnitConverter.convert(0f, AngleUnit.RADIAN, AngleUnit.DEGREE), DELTA);
		
		assertEquals(0f, UnitConverter.convert(0f, AngleUnit.RADIAN, AngleUnit.RADIAN), DELTA);
		assertEquals(1f, UnitConverter.convert(1f, AngleUnit.RADIAN, AngleUnit.RADIAN), DELTA);
		assertEquals(0f, UnitConverter.convert(0f, AngleUnit.DEGREE, AngleUnit.DEGREE), DELTA);
		assertEquals(1f, UnitConverter.convert(1f, AngleUnit.DEGREE, AngleUnit.DEGREE), DELTA);
		
		// DOUBLE
		assertEquals(Math.PI, UnitConverter.convert(180d, AngleUnit.DEGREE, AngleUnit.RADIAN), DELTA);
		assertEquals(Math.PI/2d, UnitConverter.convert(90d, AngleUnit.DEGREE, AngleUnit.RADIAN), DELTA);
		assertEquals(-Math.PI/2d, UnitConverter.convert(-90d, AngleUnit.DEGREE, AngleUnit.RADIAN), DELTA);
		assertEquals(0d, UnitConverter.convert(0d, AngleUnit.DEGREE, AngleUnit.RADIAN), DELTA);
		
		assertEquals(180d, UnitConverter.convert(Math.PI, AngleUnit.RADIAN, AngleUnit.DEGREE), DELTA);
		assertEquals(90d, UnitConverter.convert(Math.PI/2d, AngleUnit.RADIAN, AngleUnit.DEGREE), DELTA);
		assertEquals(-90d, UnitConverter.convert(-Math.PI/2d, AngleUnit.RADIAN, AngleUnit.DEGREE), DELTA);
		assertEquals(0d, UnitConverter.convert(0d, AngleUnit.RADIAN, AngleUnit.DEGREE), DELTA);
		
		assertEquals(0d, UnitConverter.convert(0d, AngleUnit.RADIAN, AngleUnit.RADIAN), DELTA);
		assertEquals(1d, UnitConverter.convert(1d, AngleUnit.RADIAN, AngleUnit.RADIAN), DELTA);
		assertEquals(0d, UnitConverter.convert(0d, AngleUnit.DEGREE, AngleUnit.DEGREE), DELTA);
		assertEquals(1d, UnitConverter.convert(1d, AngleUnit.DEGREE, AngleUnit.DEGREE), DELTA);
		
		
		// ############# SPEED ###############
		
		// FLOAT
		assertEquals(3.6, UnitConverter.convert(1f, SpeedUnit.METER_PER_SEC, SpeedUnit.KILOMETER_PER_HOUR), DELTA);
		assertEquals(-3.6, UnitConverter.convert(-1f, SpeedUnit.METER_PER_SEC, SpeedUnit.KILOMETER_PER_HOUR), DELTA);
		assertEquals(0f, UnitConverter.convert(0f, SpeedUnit.METER_PER_SEC, SpeedUnit.KILOMETER_PER_HOUR), DELTA);
		
		assertEquals((float) 1/3.6f, UnitConverter.convert(1f, SpeedUnit.KILOMETER_PER_HOUR, SpeedUnit.METER_PER_SEC), DELTA);
		assertEquals((float) 1/-3.6f, UnitConverter.convert(-1f, SpeedUnit.KILOMETER_PER_HOUR, SpeedUnit.METER_PER_SEC), DELTA);
		assertEquals(0f, UnitConverter.convert(0f, SpeedUnit.KILOMETER_PER_HOUR, SpeedUnit.METER_PER_SEC), DELTA);
		
		assertEquals(0f, UnitConverter.convert(0f, SpeedUnit.KILOMETER_PER_HOUR, SpeedUnit.KILOMETER_PER_HOUR), DELTA);
		assertEquals(1f, UnitConverter.convert(1f, SpeedUnit.KILOMETER_PER_HOUR, SpeedUnit.KILOMETER_PER_HOUR), DELTA);
		assertEquals(0f, UnitConverter.convert(0f, SpeedUnit.METER_PER_SEC, SpeedUnit.METER_PER_SEC), DELTA);
		assertEquals(1f, UnitConverter.convert(1f, SpeedUnit.METER_PER_SEC, SpeedUnit.METER_PER_SEC), DELTA);
		
		// DOUBLE
		assertEquals(3.6, UnitConverter.convert(1d, SpeedUnit.METER_PER_SEC, SpeedUnit.KILOMETER_PER_HOUR), DELTA);
		assertEquals(-3.6, UnitConverter.convert(-1d, SpeedUnit.METER_PER_SEC, SpeedUnit.KILOMETER_PER_HOUR), DELTA);
		assertEquals(0d, UnitConverter.convert(0d, SpeedUnit.METER_PER_SEC, SpeedUnit.KILOMETER_PER_HOUR), DELTA);
		
		assertEquals(1/3.6, UnitConverter.convert(1d, SpeedUnit.KILOMETER_PER_HOUR, SpeedUnit.METER_PER_SEC), DELTA);
		assertEquals(-1/3.6, UnitConverter.convert(-1d, SpeedUnit.KILOMETER_PER_HOUR, SpeedUnit.METER_PER_SEC), DELTA);
		assertEquals(0d, UnitConverter.convert(0d, SpeedUnit.KILOMETER_PER_HOUR, SpeedUnit.METER_PER_SEC), DELTA);
		
		assertEquals(0d, UnitConverter.convert(0d, SpeedUnit.KILOMETER_PER_HOUR, SpeedUnit.KILOMETER_PER_HOUR), DELTA);
		assertEquals(1d, UnitConverter.convert(1d, SpeedUnit.KILOMETER_PER_HOUR, SpeedUnit.KILOMETER_PER_HOUR), DELTA);
		assertEquals(0d, UnitConverter.convert(0d, SpeedUnit.METER_PER_SEC, SpeedUnit.METER_PER_SEC), DELTA);
		assertEquals(1d, UnitConverter.convert(1d, SpeedUnit.METER_PER_SEC, SpeedUnit.METER_PER_SEC), DELTA);
		
		// ############# ROTATIONAL SPEED ###############
		
		// FLOAT
		assertEquals(180.0/Math.PI, UnitConverter.convert(1f, RotationalSpeedUnit.RADIANS_PER_SEC, RotationalSpeedUnit.DEGREES_PER_SEC), DELTA);
		assertEquals(-180.0/Math.PI, UnitConverter.convert(-1f, RotationalSpeedUnit.RADIANS_PER_SEC, RotationalSpeedUnit.DEGREES_PER_SEC), DELTA);
		assertEquals(0f, UnitConverter.convert(0f, RotationalSpeedUnit.RADIANS_PER_SEC, RotationalSpeedUnit.DEGREES_PER_SEC), DELTA);
		
		assertEquals((float) Math.PI/180.0, UnitConverter.convert(1f, RotationalSpeedUnit.DEGREES_PER_SEC, RotationalSpeedUnit.RADIANS_PER_SEC), DELTA);
		assertEquals((float) -Math.PI/180.0, UnitConverter.convert(-1f, RotationalSpeedUnit.DEGREES_PER_SEC, RotationalSpeedUnit.RADIANS_PER_SEC), DELTA);
		assertEquals(0f, UnitConverter.convert(0f, RotationalSpeedUnit.DEGREES_PER_SEC, RotationalSpeedUnit.RADIANS_PER_SEC), DELTA);
		
		assertEquals(0f, UnitConverter.convert(0f, RotationalSpeedUnit.DEGREES_PER_SEC, RotationalSpeedUnit.DEGREES_PER_SEC), DELTA);
		assertEquals(1f, UnitConverter.convert(1f, RotationalSpeedUnit.DEGREES_PER_SEC, RotationalSpeedUnit.DEGREES_PER_SEC), DELTA);
		assertEquals(0f, UnitConverter.convert(0f, RotationalSpeedUnit.RADIANS_PER_SEC, RotationalSpeedUnit.RADIANS_PER_SEC), DELTA);
		assertEquals(1f, UnitConverter.convert(1f, RotationalSpeedUnit.RADIANS_PER_SEC, RotationalSpeedUnit.RADIANS_PER_SEC), DELTA);
		
		// DOUBLE
		assertEquals(180.0/Math.PI, UnitConverter.convert(1d, RotationalSpeedUnit.RADIANS_PER_SEC, RotationalSpeedUnit.DEGREES_PER_SEC), DELTA);
		assertEquals(-180.0/Math.PI, UnitConverter.convert(-1d, RotationalSpeedUnit.RADIANS_PER_SEC, RotationalSpeedUnit.DEGREES_PER_SEC), DELTA);
		assertEquals(0d, UnitConverter.convert(0d, RotationalSpeedUnit.RADIANS_PER_SEC, RotationalSpeedUnit.DEGREES_PER_SEC), DELTA);
		
		assertEquals(Math.PI/180.0, UnitConverter.convert(1d, RotationalSpeedUnit.DEGREES_PER_SEC, RotationalSpeedUnit.RADIANS_PER_SEC), DELTA);
		assertEquals(-Math.PI/180.0, UnitConverter.convert(-1d, RotationalSpeedUnit.DEGREES_PER_SEC, RotationalSpeedUnit.RADIANS_PER_SEC), DELTA);
		assertEquals(0d, UnitConverter.convert(0d, RotationalSpeedUnit.DEGREES_PER_SEC, RotationalSpeedUnit.RADIANS_PER_SEC), DELTA);
		
		assertEquals(0d, UnitConverter.convert(0d, RotationalSpeedUnit.DEGREES_PER_SEC, RotationalSpeedUnit.DEGREES_PER_SEC), DELTA);
		assertEquals(1d, UnitConverter.convert(1d, RotationalSpeedUnit.DEGREES_PER_SEC, RotationalSpeedUnit.DEGREES_PER_SEC), DELTA);
		assertEquals(0d, UnitConverter.convert(0d, RotationalSpeedUnit.RADIANS_PER_SEC, RotationalSpeedUnit.RADIANS_PER_SEC), DELTA);
		assertEquals(1d, UnitConverter.convert(1d, RotationalSpeedUnit.RADIANS_PER_SEC, RotationalSpeedUnit.RADIANS_PER_SEC), DELTA);
		
		
		// ############# TIME ###############
		
		assertEquals(1L, UnitConverter.convert(1L, TimeUnit.MICROSECONDS, TimeUnit.MICROSECONDS));
		assertEquals(1000L, UnitConverter.convert(1L, TimeUnit.MILLISECONDS, TimeUnit.MICROSECONDS));
		assertEquals(1000L * 1000L, UnitConverter.convert(1L, TimeUnit.SECONDS, TimeUnit.MICROSECONDS));
		assertEquals(1000L * 1000L * 60L, UnitConverter.convert(1L, TimeUnit.MINUTES, TimeUnit.MICROSECONDS));
		assertEquals(1000L * 1000L * 60L * 60L, UnitConverter.convert(1L, TimeUnit.HOURS, TimeUnit.MICROSECONDS));
		assertEquals(1000L * 1000L * 60L * 60L * 24L, UnitConverter.convert(1L, TimeUnit.DAYS, TimeUnit.MICROSECONDS));
		
		assertEquals(0L, UnitConverter.convert(1L, TimeUnit.MICROSECONDS, TimeUnit.DAYS));
		assertEquals(0L, UnitConverter.convert(1L, TimeUnit.MILLISECONDS, TimeUnit.DAYS));
		assertEquals(0L, UnitConverter.convert(1L, TimeUnit.SECONDS, TimeUnit.DAYS));
		assertEquals(0L, UnitConverter.convert(1L, TimeUnit.MINUTES, TimeUnit.DAYS));
		assertEquals(0L, UnitConverter.convert(1L, TimeUnit.HOURS, TimeUnit.DAYS));
		assertEquals(1L, UnitConverter.convert(1L, TimeUnit.DAYS, TimeUnit.DAYS));
		
		assertEquals(1L, UnitConverter.convert(1000L * 1000L, TimeUnit.MICROSECONDS, TimeUnit.SECONDS));
		assertEquals(1L, UnitConverter.convert(1000L, TimeUnit.MILLISECONDS, TimeUnit.SECONDS));
		assertEquals(1L, UnitConverter.convert(1L, TimeUnit.SECONDS, TimeUnit.SECONDS));
		assertEquals(60L, UnitConverter.convert(1L, TimeUnit.MINUTES, TimeUnit.SECONDS));
		assertEquals(60L * 60L, UnitConverter.convert(1L, TimeUnit.HOURS, TimeUnit.SECONDS));
		assertEquals(60L * 60L * 24L, UnitConverter.convert(1L, TimeUnit.DAYS, TimeUnit.SECONDS));
	}
}
